/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "matrix.h"

void matrixIdentity(matrix_t *matrix) {
  ASSERT_NOT_NULL(matrix);
  glm_mat4_identity(matrix->internalMatrix);
}

void matrixLookAt(matrix_t *matrix,
  float x,float y,float z,
  float tx,float ty, float tz,
  float ux, float uy, float uz
) {
  ASSERT_NOT_NULL(matrix);
  glm_lookat(
    (vec3){  x,  y,  z },
    (vec3){ tx, ty, tz },
    (vec3){ ux, uy, uz },
    matrix->internalMatrix
  );
}

void matrixLook(matrix_t *matrix, 
  float x, float y, float z,
  float pitch, float yaw, float roll, 
  float ux, float uy, float uz
) {
  ASSERT_NOT_NULL(matrix);
  glm_look(
    (vec3){ x, y, z },
    (vec3){ pitch, yaw, roll },
    (vec3){ ux, uy, uz },
    matrix->internalMatrix
  );
}

void matrixPerspective(matrix_t *matrix,
  float fov, float aspect, float camNear, float camFar
) {
  ASSERT_NOT_NULL(matrix);
  glm_perspective(fov, aspect, camNear, camFar, matrix->internalMatrix);
}

void matrixOrtho(matrix_t *matrix,
  float left, float right, float bottom, float top, float camNear, float camFar
) {
  ASSERT_NOT_NULL(matrix);
  glm_ortho(left, right, bottom, top, camNear, camFar, matrix->internalMatrix);
}

void matrixTranslate(matrix_t *matrix, float x, float y, float z) {
  ASSERT_NOT_NULL(matrix);
  glm_translate(matrix->internalMatrix, (vec3){ x, y, z });
}

void matrixRotate(matrix_t *matrix, float angle, float x, float y, float z) {
  ASSERT_NOT_NULL(matrix);
  glm_rotate(matrix->internalMatrix, angle, (vec3){ x, y, z });
}

void matrixScale(matrix_t *matrix, float x, float y, float z) {
  ASSERT_NOT_NULL(matrix);
  glm_scale(matrix->internalMatrix, (vec3){ x, y, z });
}

matrix_t matrixPositionAndScale(
  float x, float y, float z,
  float pitch, float yaw, float roll,
  float scaleX, float scaleY, float scaleZ
) {
  matrix_t matrix;

  matrixIdentity(&matrix);
  matrixTranslate(&matrix, x, y, z);

  // Rotation (YZX order)
  matrixRotate(&matrix, yaw, 0, 1, 0);
  matrixRotate(&matrix, roll, 0, 0, 1);
  matrixRotate(&matrix, pitch, 1, 0, 0);

  matrixScale(&matrix, scaleX, scaleY, scaleZ);

  return matrix;
}