/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "scene.h"

void sceneInit(scene_t *scene, engine_t *engine, void *user) {
  scene->engine = engine;
  scene->assetOwner = assetManagerHolderCreate(&engine->assetManager);
  scene->user = user;
  scene->onLoaded = NULL;
  scene->onRender = NULL;
  scene->loaded = false;
}

float sceneUpdate(scene_t *scene) {
  // First, lets check how loading has been going
  float n = assetManagerProgressGetForHolder(
    &scene->engine->assetManager, scene->assetOwner
  );

  // TODO: Loading screen
  if(n < 1.0f) return n;

  // Fire event
  if(!scene->loaded) {
    scene->loaded = true;
    if(scene->onLoaded != NULL) scene->onLoaded(scene);
  }

  // Fire callback
  if(scene->onRender != NULL) scene->onRender(scene);

  return n;
}

void sceneDispose(scene_t *scene) {
  assetManagerHolderRelease(&scene->engine->assetManager, scene->assetOwner);
}